function [X, Y, MU] = contour_display(A, B, epsln, X, Y, gamma, h, c, lw)
% contour_display(A, B, epsln, X, Y, gamma)
%   is an auxiliary function to draw the contour plot of the real 
%   pseudospectrum (if gamma == 0) and superset (if gamma in (0,1]), 
%   over the grid defined by X, Y.  
%
%   With optional parameter h (figure handle) and c (RGB matrix) provided,
%   the contour will be plotted on figure h, using color c.

%   Revised:    Jun. 13, 2016.

if nargin < 9 || isempty(lw), lw = 2; end % default black color
if nargin < 8 || isempty(c), c =  [0,0,0]; end % default black color
if nargin < 7, h = figure; end % default figure

if gamma == 0 % plot real pseudospectrum
    [X, Y, MU] = mu_grid(A, B, X, Y); % evaluate mu values on the grid
    figure(h); hold on;
    contour(X, Y, MU, [epsln epsln], 'linewidth', lw, 'color', c); 
else
    [X, Y, MU] = g_grid(A, B, gamma, X, Y);
    figure(h); hold on;
    contour(X, Y, MU, [epsln epsln], 'linewidth', lw, 'color', c);
end
drawnow;

return

% ================== AUXILIARY FUNCTION (1) =======================

function [X, Y, MU] = g_grid(A, B, gamma, X, Y0)
%   [X, Y, MU] = mu_grid(A, B, X0, Y0) produces the mu value the 
%   matrix pencil (A, B), over the grid (X, Y0).
%
%   On output, the grid (X Y), as well as MU, are symmetrized w.r.t 
%   the real axis.


if ~isrow(Y0), Y0 = Y0'; end; 

Y_pos = Y0(Y0 > 0); % positive part of vertical grid
Y_pos = [0 Y_pos]; % add real line

Y_neg = Y0(Y0 < 0); % negative part of vertical grid
Y_neg = [Y_neg 0]; % add real line

if max(Y_pos) >= -min(Y_neg) % positive part has larger range
    Y = Y_pos;              % use positive part to define grid
else
    Y = -flip(Y_neg);       % else, reflect neg part to define
end

MU = zeros(length(Y),length(X)); 

for ii = 1 : length(X)
    alpha0 = X(ii);
    parfor jj = 1 : length(Y)
        beta0 = Y(jj);
        h = [A-alpha0*B, -beta0*gamma*B; beta0/gamma*B, A-alpha0*B];
        s = svd(h);
        MU(jj, ii) = s(end-1); 
    end
end

% Reflect w.r.t. real line
Y = [-flip(Y), Y(2:end)];
MU = [flipud(MU); MU(2:end,:)];

return


% -----------------------------------------------------------------

function [X, Y, MU] = mu_grid(A, B, X, Y0)
%   [X, Y, MU] = mu_grid(A, B, X0, Y0) produces the mu value the 
%   matrix pencil (A, B), over the grid (X, Y0).
%
%   On output, the grid (X Y), as well as MU, are symmetrized w.r.t 
%   the real axis.

if ~isrow(Y0), Y0 = Y0'; end; 

Y_pos = Y0(Y0 > 0); % positive part of vertical grid
Y_pos = [0 Y_pos]; % add real line

Y_neg = Y0(Y0 < 0); % negative part of vertical grid
Y_neg = [Y_neg 0]; % add real line

if max(Y_pos) > -min(Y_neg) % positive part has larger range
    Y = Y_pos;              % use positive part to define grid
else
    Y = flip(-Y_neg);       % else, reflect neg part to define
end

% Evaluate the mu value
MU = zeros(length(Y),length(X)); 
for ii = 1 : length(X)  
    alpha0 = X(ii);
    parfor jj = 1 : length(Y)
        beta0 = Y(jj);
        MU(jj, ii) = eval_mu(A, B, alpha0, beta0); 
    end
end

% Reflect w.r.t. real line
Y = [-flip(Y), Y(2:end)];
MU = [flipud(MU); MU(2:end,:)];

return


