function [examples, infos, zvecs] = example_subspace_comparison(epsln)
%   Performance comparison with Rostami's algorithm 
%   epsln = 0.01, .1, 1
%
%   IMPORTANT:  For comparison convenience, please slightly modify
%               the function real_abscissa_initial.m, available from
%               http://users.wpi.edu/~mwu/webpage_code/pseudo/,
%               to take the variable 'keig' (defined in line 12) 
%               as the third input argument of the function
%               (see line 58 below).
% 
%   REMARKS:    To get more accurate timing statistics, please set 
%               NB_SAMPLES in line 13 to be a relative large number.
%
%               The simulation may take up to several hours,
%               depending on your computational resources.
%
%   Revised:    Jun. 13, 2016.

NB_SAMPLES = 1;     % repeat runs for timing  
neigs_tolosa = 150; % number of eigs used for tolosa example

examples = {
'pde_demo(2961)',
'rdbrusselator_demo(3200)',
'tolosa_demo(4000)',
'tubular_demo(1000)'
};


result = [];
for i = 1 : length(examples)
    eval(['A=',examples{i},';']);

    if strcmp(examples{i}, 'tolosa_demo(4000)')
        opts.neigs = neigs_tolosa;
        neigs = neigs_tolosa;
        SAMPLES = 1;
    else
        opts.neigs = 20;
        neigs = 20; 
        SAMPLES = NB_SAMPLES;
    end

    tic;    % RPSAS ALGORITHM
    for kk = 1:SAMPLES
        rng('default');
        [alpha, info] = rpsas(A, epsln, opts);
    end
    t = toc/SAMPLES;

    z = info.z; beta = imag(z(end));
    mu = eval_mu(A, speye(size(A)), alpha, beta);

    tic;    % ROSTAMI'S ALGORITHM
    for kk = 1:SAMPLES
        rng('default'); % generate predictable rand for eigs
        [X,Y,U,V,zvec] = real_abscissa_initial(A, epsln, neigs);
    end
    t1 = toc/SAMPLES;

    z1 = zvec(end); 
    mu1 = eval_mu(A, speye(size(A)), real(z1), imag(z1));

    result = [result;
                alpha+1j*beta, info.its, mu, t, z1, length(zvec), mu1, t1];
    infos{i} = info;
    zvecs{i} = zvec; 

    print_rst(result(i,:), i-1);

end

print_rst(result, 0);

save(['./examples_paper/results/subspace_comparison_epsln_',num2str(epsln,'%1.0E'),'.mat'], 'result');

return


% =========================================================================
% SUBFUNCTION
% =========================================================================

function xx = print_rst(result, inc)
fprintf(' RPSAS: alpha, itr, timing, mu_val\n')
for i = 1 : size(result,1)
    r = result(i,:);
    fprintf('Ex%1i: %+9.8f       %2i     %4.1f  %9.8E\n', i+inc, real(r(1)), r(2) , r(4), r(3));
end

fprintf('\n\n ROSTAMI ALG: alpha, itr, timing, mu_val\n')
for i = 1 : size(result, 1)
    r = result(i,:);
    fprintf('Ex%1i: %+9.8f       %2i     %4.1f  %9.8E\n', i+inc, real(r(5)), r(6) , r(8), r(7));
end
return
