function example_subspace_grcar(ex_grcar, modeeig)
% =======================================================
%   Demo routine for the 100 * 100 grcar matrix, to show
%   performance of subspace method
%
%   ex_grcar =  true          run grcar matrix
%   otherwise                 run -grcar matrix
%
%   modeeig = true            use eigvec to expand 
%   otherwise                 use singular vector to expand 
%
%   Set doplot (line 21) true if plots are wanted.
%   ----WARNING:    the plotting may take up to several minutes,
%                   depending on your computational resources.
%                   We encourage using MATLAB parallel computation
%                   facilities, since PARFOR is called in the code.
%
%   Revised:    Jun. 13, 2016.
% =======================================================

doplot  = false;    % set to true if plots is wanted
maxit   = 30;       % maximum iteration number
tol     = 1.0E-12;  % tolerance
neigs   = 20;
rng('default');     % random number seeds, 
                    % generate predictable rand for eigs

A = grcar_demo(100); % generate coefficient matrix

if ex_grcar % Grcar problem 
    A0 = A;
    epsln = .3;
    X = linspace(1.3,3.3,40); 
    Y = linspace(0,1.5,40);
    axs = [1.3,3.3, -1.5,1.5];
else % -Grcar problem 
    A0 = -A; 
    epsln = .2;
    X = linspace(-.5,1,40); 
    Y = linspace(.5,3.5,40);
    axs = [-.5,1,.5,3.5];
end

n = length(A0); 
B0 = eye(n);

% Compute 'exact rpsa'
opts.tol = eps;
[alpha0, info0] = rpsa(A0, epsln, opts);

% Contour data
if doplot
    h = figure;
    contour_display(A0, B0, epsln, X, Y, 0, h, [0,0,0]);
    axis(axs);
end

ee0 = eig(A0);

% Initialize projection subspace, evaluate eigenvalue of largest real part
[V, e0] = rpsas_init_subspace(A0, neigs);
alpha = real(e0); 
beta = imag(e0);

hstr=[];   % record computation history 

for jj = 1 : maxit
    % ------------------------
    % STEP 1: Compute reduced problem
    % ------------------------
    opts0.alpha0 = alpha; 
    opts0.beta0 = beta; 
    opts0.tol = eps*100;
    [alpha, beta, gamma, info] = rpsa_pencil(A0*V, V, epsln, opts0);
    [~,gammao]= eval_mu(A0, eye(size(A0)), alpha, beta, epsln); % just for comparison
    hstr = [hstr; alpha, beta, gammao, gamma];

    if doplot
        hh = copyobj(h, 'legacy'); % copy original contour
        contour_display(A0*V, V, epsln, X, Y, 0, hh, [0,0,0]);
        plot(ee0,'.k', 'markersize', 12, 'linewidth', 8);
        plot(alpha,beta,'+k', 'linewidth',3,'markersize',13); 
        drawnow;
    end

    % ------------------------
    % CONVERGENCE CHECK 
    % ------------------------
    if jj > 1 && alpha - hstr(jj-1,1) < tol*abs(alpha)
         itr = jj;
         break; 
    end

    % ------------------------
    % STEP 2: Expand subspace
    % -----------------------
    [~, v, u] = second_smallest_sv(A0, B0, alpha, beta, gamma);
    v = reshape(v, n, []);
    if modeeig, v = rpsas_expand_eigvec(A0, u, v, epsln, neigs); end

    [V,~] = qr([V,v],0);
end

disp('Exact RPSA (criss-cross):');
disp(alpha0);
disp('alpha_k,  beta_k, gamma_k, AbsErr(alpha_k):')
disp([hstr(:,1:3), abs(hstr(:,1) - alpha0)]);
